{$d-,l-}
unit PackInfo;

INTERFACE

{ input:  fn    - file name
  output: ps    - packed size (size of the file)
          ups   - unpacked size
          split - splited archive indicator (multi volume)
  returns -1 in PS if file wasn't found;
          -1 in UPS if not archive file }

procedure ZipInfo(fn:string;var PS,UPS:longint;var split:boolean);
procedure RarInfo(fn:string;var PS,UPS:longint;var split:boolean);
procedure ArjInfo(fn:string;var PS,UPS:longint;var split:boolean);
procedure LzhInfo(fn:string;var PS,UPS:longint;var split:boolean);
procedure SfxInfo(fn:string;var PS,UPS:longint;var split:boolean);

IMPLEMENTATION

uses dos;
var Sfx:record
         Yes:boolean;
         Start:word;
        end;

{------ function returns total uncompressed ZIP archive size in bytes ------}
{------ doesn't work on splited archives ------}
procedure ZipInfo(fn:string;var PS,UPS:longint;var split:boolean);
 type Header=record
              ID:longint;
              Data:array[1..14] of byte;
              CS:longint;        { compressed size }
              US:longint;        { uncompressed size }
              LFName:word;
              LExField:word;
             end;
 var TotSize,PackSize:longint;
     HR:Header;
     f:file;
     res:word;
     i:longint;
     b:byte;
     srec:searchrec;
 begin
  TotSize:=0;
  Split:=false;
  PS:=-1;
  UPS:=-1;
  {$i-}
  FindFirst(fn,anyfile,srec);
   if DosError=0 then PackSize:=srec.size
                 else Exit;
  Assign(f,fn);
  FileMode:=0;
  Reset(f,1);
  if IoResult<>0 then exit;
  if Sfx.Yes then
   begin
    seek(f,Sfx.Start);
    Sfx.yes:=false;
   end;
  BlockRead(f,HR,SizeOf(HR),res);
  if (res<>SizeOf(HR)) or (HR.ID<>$04034B50) then
   begin
    Close(f);
    Exit;
   end;
  while (not eof(f)) and (HR.ID=$04034B50) do
   begin
    TotSize:=TotSize+HR.US;
    i:=FilePos(f)+HR.CS+HR.LFname+HR.LExField;
    seek(f,i);
    BlockRead(f,HR,SizeOf(HR));
   end;
  Close(f);
  {$i+}
  UPS:=TotSize;
  PS:=PackSize;
 end;
{------ function returns total uncompressed RAR file size in bytes ------}
procedure RarInfo(fn:string;var PS,UPS:longint;var split:boolean);
 type Header=array[1..7] of byte;
 const Mark_Header:Header=($52,$61,$72,$21,$1a,$07,$00);
 var totsize:longint;
     packsize:longint;
     f:file;
     HR:Header;                { Header readed }
     res:word;
     i:longint;
     multivol:boolean;
     Size:longint;
     sr:SearchRec;

 procedure Volume(fname:string);
  var i:longint;
  begin
  {$i-}
   if split then PackSize:=PackSize+sr.size;
   Assign(f,fname);
   FileMode:=0;
   Reset(f,1);
   if IoResult<>0 then Exit;
   if Sfx.Yes then
    begin
     Seek(f,sfx.start);
     Sfx.Yes:=false;
    end;
   BlockRead(f,HR,SizeOf(HR),res);
   if res<>SizeOf(HR) then
    begin
     Close(f);
     Exit;
    end;
   for i:=1 to 7 do if HR[i]<>Mark_Header[i] then
    begin
     Close(f);
     Exit;
    end;
   BlockRead(f,HR,SizeOf(HR),res);
   if (res<>SizeOf(HR)) or (HR[3]<>$73) then
    begin
     Close(f);
     Exit;
    end;
   i:=filepos(f)+HR[6]+HR[7]*$100-7;
   seek(f,i);
   while not eof(f) do
    begin
     BlockRead(f,HR,SizeOf(HR),res);
     if res<>SizeOf(HR) then
      begin
       Close(f);
       Exit;
      end;
     if HR[3]=$74 then
      begin
       Blockread(f,Size,4);
       i:=filepos(f)+HR[6]+HR[7]*$100+Size-11;
       Blockread(f,Size,4);
       if (HR[4] and 1)=0 then TotSize:=TotSize+size;
      end
     else i:=filepos(f)+HR[6]+HR[7]*$100-7;
     seek(f,i);
    end;
   Close(f);
   {$i+}
  end;

 begin
  multivol:=false;
  TotSize:=0;
  PackSize:=0;
  PS:=-1;
  UPS:=-1;
  Split:=false;
  {$i-}
  Assign(f,fn);
  FileMode:=0;
  Reset(f,1);
  if IoResult<>0 then Exit;
  if Sfx.Yes then Seek(f,Sfx.Start);
  BlockRead(f,HR,SizeOf(HR),res);
  if res<>SizeOf(HR) then
   begin
    Close(f);
    Exit;
   end;
  for i:=1 to 7 do if HR[i]<>Mark_Header[i] then
   begin
    Close(f);
    Exit;
   end;
  BlockRead(f,HR,SizeOf(HR),res);
  if (res<>SizeOf(HR)) or (HR[3]<>$73) then
   begin
    Close(f);
    Exit;
   end;
  Close(f);
  if (HR[4] and 1)=0 then
   begin
    FindFirst(fn,anyfile,sr);
    PackSize:=sr.size;
    Volume(fn);
   end
  else
   begin
    split:=true;
    fn:=copy(fn,1,pos('.',fn));
    findfirst(fn+'*',anyfile,sr);
    while doserror=0 do
     begin
      delete(sr.name,1,pos('.',sr.name));
      Volume(fn+sr.name);
      findnext(sr);
     end;
   end;
  {$i+}
  UPS:=TotSize;
  PS:=PackSize;
 end;
{------ function returns total uncompressed ARJ archive size in bytes ------}
procedure ArjInfo(Fn:string;var PS,UPS:longint;var Split:boolean);
 var f:file;
     TotSize:longint;
     PackSize:longint;
     res:word;
     HR:record
         ID:word;         { ID bytes }
         S:word;          { Size of header }
         Data1:array[1..4] of byte;
         Flag:byte;
         Data2:byte;
         Ftype:byte;
         Data3:array[1..5] of byte;
         CS,US:longint;
        end;
     A:word;
     i:longint;
     multivol:boolean;
     sr:searchrec;
     b:byte;

 procedure Volume(FName:string);
  var i:longint;
  begin
   {$i-}
   if split then PackSize:=PackSize+sr.size;
   Assign(f,FName);
   FileMode:=0;
   Reset(f,1);
   if ioresult<>0 then exit;
   if Sfx.Yes then
    begin
     Seek(f,Sfx.Start);
     Sfx.Yes:=false;
    end;
   BlockRead(f,HR,SizeOf(HR),res);
   if (res<>SizeOf(HR)) or (HR.ID<>$EA60) or (HR.Ftype<>2) then
    begin
     Close(f);
     Exit;
    end;
   i:=FilePos(f)-SizeOf(HR)+HR.S+8;
   Seek(f,i);
   BlockRead(f,A,2);
   if A<>0 then
    Seek(f,FilePos(f)+A+4);
   while not Eof(f) do
    begin
     BlockRead(f,HR,SizeOf(HR),res);
     if res<>SizeOf(HR) then
      begin
       Close(f);
       Exit;
      end;
     if HR.ID=$EA60 then
      begin
       TotSize:=TotSize+HR.US;
       i:=FilePos(f)-SizeOf(HR)+HR.S+8;
       Seek(f,i);
       BlockRead(f,A,2);
       if A<>0 then
        Seek(f,FilePos(f)+A+4);
       Seek(f,FilePos(f)+HR.CS);
      end;
    end;
   Close(f);
   {$i+}
  end;

 begin
  multivol:=false;
  TotSize:=0;
  PackSize:=0;
  Split:=false;
  PS:=-1;
  UPS:=-1;
  {$i-}
  Assign(f,fn);
  FileMode:=0;
  Reset(f,1);
  if IoResult<>0 then Exit;
  if Sfx.Yes then Seek(f,Sfx.Start);
  BlockRead(f,HR,SizeOf(HR),res);
  Close(f);
  if (res<>SizeOf(HR)) or (HR.ID<>$EA60) or (HR.Ftype<>2) then Exit;
  if (HR.Flag and 4)=0 then
   begin
    FindFirst(fn,anyfile,sr);
    PackSize:=sr.size;
    Volume(fn);
   end
  else
   begin
    split:=true;
    fn:=copy(fn,1,pos('.',fn));
    findfirst(fn+'*',anyfile,sr);
    while doserror=0 do
     begin
      delete(sr.name,1,pos('.',sr.name));
      Volume(fn+sr.name);
      findnext(sr);
     end;
   end;
  {$i+}
  UPS:=TotSize;
  PS:=PackSize;
 end;
{------ function returns total uncompressed LZH archive size in bytes ------}
procedure LzhInfo(Fn:string;var PS,UPS:longint;var split:boolean);
 var f:file;
     HR:record
         S:byte;                { header size }
         Data1:byte;
         ID1:array[1..3] of char;
         ID2:array[1..2] of char;
         CS:longint;
         US:longint;
         Data2:array[1..6] of byte;
         FnLength:byte;
        end;
     i:longint;
     TotSize:longint;
     PackSize:longint;
     res:word;
     sr:searchrec;
 begin
  TotSize:=0;
  PackSize:=0;
  PS:=-1;
  UPS:=-1;
  split:=false;
  {$i-}
  FindFirst(fn,anyfile,sr);
  if DosError=0 then PackSize:=sr.size;
  Assign(f,fn);
  FileMode:=0;
  Reset(f,1);
  if IoResult<>0 then exit;
  if Sfx.Yes then
   begin
    seek(f,Sfx.Start);
    Sfx.yes:=false;
   end;
  BlockRead(f,HR,SizeOf(HR),res);
  while not eof(f) do
   begin
    if (res<>SizeOf(HR)) or ((HR.ID1<>'-lh') and (HR.ID1<>'-lz')) or
       (HR.ID2[2]<>'-') then
     begin
      Close(f);
      Exit;
     end;
    TotSize:=TotSize+HR.US;
    i:=FilePos(f)-SizeOf(HR)+HR.S+HR.CS+2;
    seek(f,i);
    BlockRead(f,HR,SizeOf(HR),res);
   end;
  Close(f);
  {$i+}
  UPS:=TotSize;
  PS:=PackSize;
 end;
{------ function returns total uncompressed SFX archive size in bytes ------}
procedure SfxInfo(Fn:string;var PS,UPS:longint;var split:boolean);
 var f:file;
     ExeHeader:record
                ID:array[1..2] of char;
                LastPageBytes:word;
                Pages:word;
               end;
     HR:record
         ID1:word;
         ID2:word;              { Header length }
        end;
     HLzh:record
           D1:word;
           ID1:array[1..3] of char;
           ID2:array[1..2] of char;
          end;
     res:word;
 begin
  UPS:=-1;
  PS:=-1;
  Split:=false;
  {$i-}
  Assign(f,Fn);
  FileMode:=0;
  Reset(f,1);
  if ioresult<>0 then Exit;
  BlockRead(f,ExeHeader,SizeOf(ExeHeader),res);
  if (Res<>SizeOf(ExeHeader)) or (ExeHeader.ID<>'MZ') then
   begin
    Close(f);
    Exit;
   end;
  seek(f,(ExeHeader.Pages-1)*512+ExeHeader.LastPageBytes);
  if ioresult<>0 then
   begin
    Close(f);
    Exit;
   end;
  BlockRead(f,HR,SizeOf(HR),res);
  Close(f);
  if res<>SizeOf(HR) then Exit;
  Sfx.Yes:=true;
  Sfx.Start:=(ExeHeader.Pages-1)*512+ExeHeader.LastPageBytes;
  if (HR.ID1=$EA60) or (HR.ID2=$EA60) then
   begin
    if HR.ID2=$EA60 then Sfx.Start:=Sfx.Start+2;
    ArjInfo(Fn,PS,UPS,Split);
   end
  else
   if (HR.ID1=$4B50) and (HR.ID2=$0403) then ZipInfo(Fn,PS,UPS,Split)
   else
    if (HR.ID1=$6152) and (HR.ID2=$2172) then RarInfo(Fn,PS,UPS,Split)
    else
     begin
      Assign(f,Fn);
      FileMode:=0;
      Reset(f,1);
      Seek(f,sfx.start);
      BlockRead(f,HLzh,SizeOf(HLzh),res);
      Close(f);
      if (res=SizeOf(HLzh)) and
         (((HLzh.ID1='-lh') or (HLzh.ID1='-lz')) and (HLzh.ID2[2]='-')) then
        LzhInfo(Fn,PS,UPS,Split)
      else Sfx.Yes:=false;
     end;
  {$i+}
 end;

begin
 Sfx.Yes:=false;
end.